@@ -26,11 +26,12 @@ import static com.google.common.jimfs.PathNormalization.CASE_FOLD_ASCII;
 import static com.google.common.jimfs.PathNormalization.NFC;
 import static com.google.common.jimfs.PathNormalization.NFD;
 
+import com.google.common.base.MoreObjects;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
-
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import java.nio.channels.FileChannel;
 import java.nio.file.FileSystem;
 import java.nio.file.InvalidPathException;
@@ -44,31 +45,30 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.regex.Pattern;
-
-import javax.annotation.Nullable;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * Immutable configuration for an in-memory file system. A {@code Configuration} is passed to a
- * method in {@link Jimfs} such as {@link Jimfs#newFileSystem(Configuration)} to create a new
- * {@link FileSystem} instance.
+ * method in {@link Jimfs} such as {@link Jimfs#newFileSystem(Configuration)} to create a new {@link
+ * FileSystem} instance.
  *
  * @author Colin Decker
  */
 public final class Configuration {
 
   /**
-   * <p>Returns the default configuration for a UNIX-like file system. A file system created with
-   * this configuration:
+   * Returns the default configuration for a UNIX-like file system. A file system created with this
+   * configuration:
    *
    * <ul>
    *   <li>uses {@code /} as the path name separator (see {@link PathType#unix()} for more
-   *   information on the path format)</li>
-   *   <li>has root {@code /} and working directory {@code /work}</li>
-   *   <li>performs case-sensitive file lookup</li>
-   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to
-   *   avoid overhead for unneeded attributes</li>
-   *   <li>supports hard links, symbolic links, {@link SecureDirectoryStream} and
-   *   {@link FileChannel}</li>
+   *       information on the path format)
+   *   <li>has root {@code /} and working directory {@code /work}
+   *   <li>performs case-sensitive file lookup
+   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to avoid
+   *       overhead for unneeded attributes
+   *   <li>supports hard links, symbolic links, {@link SecureDirectoryStream} and {@link
+   *       FileChannel}
    * </ul>
    *
    * <p>To create a modified version of this configuration, such as to include the full set of UNIX
@@ -89,6 +89,7 @@ public final class Configuration {
   private static final class UnixHolder {
     private static final Configuration UNIX =
         Configuration.builder(PathType.unix())
+            .setDisplayName("Unix")
             .setRoots("/")
             .setWorkingDirectory("/work")
             .setAttributeViews("basic")
@@ -97,7 +98,7 @@ public final class Configuration {
   }
 
   /**
-   * <p>Returns the default configuration for a Mac OS X-like file system.
+   * Returns the default configuration for a Mac OS X-like file system.
    *
    * <p>The primary differences between this configuration and the default {@link #unix()}
    * configuration are that this configuration does Unicode normalization on the display and
@@ -107,18 +108,18 @@ public final class Configuration {
    *
    * <ul>
    *   <li>uses {@code /} as the path name separator (see {@link PathType#unix()} for more
-   *   information on the path format)</li>
-   *   <li>has root {@code /} and working directory {@code /work}</li>
-   *   <li>does Unicode normalization on paths, both for lookup and for {@code Path} objects</li>
-   *   <li>does case-insensitive (for ASCII characters only) lookup</li>
-   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to
-   *   avoid overhead for unneeded attributes</li>
-   *   <li>supports hard links, symbolic links and {@link FileChannel}</li>
+   *       information on the path format)
+   *   <li>has root {@code /} and working directory {@code /work}
+   *   <li>does Unicode normalization on paths, both for lookup and for {@code Path} objects
+   *   <li>does case-insensitive (for ASCII characters only) lookup
+   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to avoid
+   *       overhead for unneeded attributes
+   *   <li>supports hard links, symbolic links and {@link FileChannel}
    * </ul>
    *
    * <p>To create a modified version of this configuration, such as to include the full set of UNIX
-   * file attribute views or to use full Unicode case insensitivity,
-   * {@linkplain #toBuilder() create a builder}.
+   * file attribute views or to use full Unicode case insensitivity, {@linkplain #toBuilder() create
+   * a builder}.
    *
    * <p>Example:
    *
@@ -136,6 +137,7 @@ public final class Configuration {
   private static final class OsxHolder {
     private static final Configuration OS_X =
         unix().toBuilder()
+            .setDisplayName("OSX")
             .setNameDisplayNormalization(NFC) // matches JDK 1.7u40+ behavior
             .setNameCanonicalNormalization(NFD, CASE_FOLD_ASCII) // NFD is default in HFS+
             .setSupportedFeatures(LINKS, SYMBOLIC_LINKS, FILE_CHANNEL)
@@ -143,24 +145,24 @@ public final class Configuration {
   }
 
   /**
-   * <p>Returns the default configuration for a Windows-like file system. A file system created
-   * with this configuration:
+   * Returns the default configuration for a Windows-like file system. A file system created with
+   * this configuration:
    *
    * <ul>
    *   <li>uses {@code \} as the path name separator and recognizes {@code /} as a separator when
-   *   parsing paths (see {@link PathType#windows()} for more information on path format)</li>
-   *   <li>has root {@code C:\} and working directory {@code C:\work}</li>
-   *   <li>performs case-insensitive (for ASCII characters only) file lookup</li>
+   *       parsing paths (see {@link PathType#windows()} for more information on path format)
+   *   <li>has root {@code C:\} and working directory {@code C:\work}
+   *   <li>performs case-insensitive (for ASCII characters only) file lookup
    *   <li>creates {@code Path} objects that use case-insensitive (for ASCII characters only)
-   *   equality</li>
-   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to
-   *   avoid overhead for unneeded attributes</li>
-   *   <li>supports hard links, symbolic links and {@link FileChannel}</li>
+   *       equality
+   *   <li>supports only the {@linkplain BasicFileAttributeView basic} file attribute view, to avoid
+   *       overhead for unneeded attributes
+   *   <li>supports hard links, symbolic links and {@link FileChannel}
    * </ul>
    *
    * <p>To create a modified version of this configuration, such as to include the full set of
-   * Windows file attribute views or to use full Unicode case insensitivity,
-   * {@linkplain #toBuilder() create a builder}.
+   * Windows file attribute views or to use full Unicode case insensitivity, {@linkplain
+   * #toBuilder() create a builder}.
    *
    * <p>Example:
    *
@@ -178,6 +180,7 @@ public final class Configuration {
   private static final class WindowsHolder {
     private static final Configuration WINDOWS =
         Configuration.builder(PathType.windows())
+            .setDisplayName("Windows")
             .setRoots("C:\\")
             .setWorkingDirectory("C:\\work")
             .setNameCanonicalNormalization(CASE_FOLD_ASCII)
@@ -194,8 +197,8 @@ public final class Configuration {
    * returned; if the operating system is Mac OS X, {@link Configuration#osX()} is returned;
    * otherwise, {@link Configuration#unix()} is returned.
    *
-   * <p>This is the configuration used by the {@code Jimfs.newFileSystem} methods that do not take
-   * a {@code Configuration} parameter.
+   * <p>This is the configuration used by the {@code Jimfs.newFileSystem} methods that do not take a
+   * {@code Configuration} parameter.
    *
    * @since 1.1
    */
@@ -211,9 +214,7 @@ public final class Configuration {
     }
   }
 
-  /**
-   * Creates a new mutable {@link Configuration} builder using the given path type.
-   */
+  /** Creates a new mutable {@link Configuration} builder using the given path type. */
   public static Builder builder(PathType pathType) {
     return new Builder(pathType);
   }
@@ -233,6 +234,7 @@ public final class Configuration {
   final ImmutableSet<String> attributeViews;
   final ImmutableSet<AttributeProvider> attributeProviders;
   final ImmutableMap<String, Object> defaultAttributeValues;
+  final FileTimeSource fileTimeSource;
 
   // Watch service
   final WatchServiceConfiguration watchServiceConfig;
@@ -241,10 +243,9 @@ public final class Configuration {
   final ImmutableSet<String> roots;
   final String workingDirectory;
   final ImmutableSet<Feature> supportedFeatures;
+  private final String displayName;
 
-  /**
-   * Creates an immutable configuration object from the given builder.
-   */
+  /** Creates an immutable configuration object from the given builder. */
   private Configuration(Builder builder) {
     this.pathType = builder.pathType;
     this.nameDisplayNormalization = builder.nameDisplayNormalization;
@@ -262,10 +263,52 @@ public final class Configuration {
         builder.defaultAttributeValues == null
             ? ImmutableMap.<String, Object>of()
             : ImmutableMap.copyOf(builder.defaultAttributeValues);
+    this.fileTimeSource = builder.fileTimeSource;
     this.watchServiceConfig = builder.watchServiceConfig;
     this.roots = builder.roots;
     this.workingDirectory = builder.workingDirectory;
     this.supportedFeatures = builder.supportedFeatures;
+    this.displayName = builder.displayName;
+  }
+
+  @Override
+  public String toString() {
+    if (displayName != null) {
+      return MoreObjects.toStringHelper(this).addValue(displayName).toString();
+    }
+    MoreObjects.ToStringHelper helper =
+        MoreObjects.toStringHelper(this)
+            .add("pathType", pathType)
+            .add("roots", roots)
+            .add("supportedFeatures", supportedFeatures)
+            .add("workingDirectory", workingDirectory);
+    if (!nameDisplayNormalization.isEmpty()) {
+      helper.add("nameDisplayNormalization", nameDisplayNormalization);
+    }
+    if (!nameCanonicalNormalization.isEmpty()) {
+      helper.add("nameCanonicalNormalization", nameCanonicalNormalization);
+    }
+    helper
+        .add("pathEqualityUsesCanonicalForm", pathEqualityUsesCanonicalForm)
+        .add("blockSize", blockSize)
+        .add("maxSize", maxSize);
+    if (maxCacheSize != Builder.DEFAULT_MAX_CACHE_SIZE) {
+      helper.add("maxCacheSize", maxCacheSize);
+    }
+    if (!attributeViews.isEmpty()) {
+      helper.add("attributeViews", attributeViews);
+    }
+    if (!attributeProviders.isEmpty()) {
+      helper.add("attributeProviders", attributeProviders);
+    }
+    if (!defaultAttributeValues.isEmpty()) {
+      helper.add("defaultAttributeValues", defaultAttributeValues);
+    }
+    helper.add("fileTimeSource", fileTimeSource);
+    if (watchServiceConfig != WatchServiceConfiguration.DEFAULT) {
+      helper.add("watchServiceConfig", watchServiceConfig);
+    }
+    return helper.toString();
   }
 
   /**
@@ -275,9 +318,7 @@ public final class Configuration {
     return new Builder(this);
   }
 
-  /**
-   * Mutable builder for {@link Configuration} objects.
-   */
+  /** Mutable builder for {@link Configuration} objects. */
   public static final class Builder {
 
     /** 8 KB. */
@@ -304,6 +345,7 @@ public final class Configuration {
     private ImmutableSet<String> attributeViews = ImmutableSet.of();
     private Set<AttributeProvider> attributeProviders = null;
     private Map<String, Object> defaultAttributeValues;
+    private FileTimeSource fileTimeSource = SystemFileTimeSource.INSTANCE;
 
     // Watch service
     private WatchServiceConfiguration watchServiceConfig = WatchServiceConfiguration.DEFAULT;
@@ -312,6 +354,7 @@ public final class Configuration {
     private ImmutableSet<String> roots = ImmutableSet.of();
     private String workingDirectory;
     private ImmutableSet<Feature> supportedFeatures = ImmutableSet.of();
+    private String displayName;
 
     private Builder(PathType pathType) {
       this.pathType = checkNotNull(pathType);
@@ -334,16 +377,19 @@ public final class Configuration {
           configuration.defaultAttributeValues.isEmpty()
               ? null
               : new HashMap<>(configuration.defaultAttributeValues);
+      this.fileTimeSource = configuration.fileTimeSource;
       this.watchServiceConfig = configuration.watchServiceConfig;
       this.roots = configuration.roots;
       this.workingDirectory = configuration.workingDirectory;
       this.supportedFeatures = configuration.supportedFeatures;
+      // displayName intentionally not copied from the Configuration
     }
 
     /**
      * Sets the normalizations that will be applied to the display form of filenames. The display
      * form is used in the {@code toString()} of {@code Path} objects.
      */
+    @CanIgnoreReturnValue
     public Builder setNameDisplayNormalization(PathNormalization first, PathNormalization... more) {
       this.nameDisplayNormalization = checkNormalizations(Lists.asList(first, more));
       return this;
@@ -354,6 +400,7 @@ public final class Configuration {
      * file system. The canonical form is used to determine the equality of two filenames when
      * performing a file lookup.
      */
+    @CanIgnoreReturnValue
     public Builder setNameCanonicalNormalization(
         PathNormalization first, PathNormalization... more) {
       this.nameCanonicalNormalization = checkNormalizations(Lists.asList(first, more));
@@ -408,6 +455,7 @@ public final class Configuration {
      *
      * <p>The default is false.
      */
+    @CanIgnoreReturnValue
     public Builder setPathEqualityUsesCanonicalForm(boolean useCanonicalForm) {
       this.pathEqualityUsesCanonicalForm = useCanonicalForm;
       return this;
@@ -419,6 +467,7 @@ public final class Configuration {
      *
      * <p>The default is 8192 bytes (8 KB).
      */
+    @CanIgnoreReturnValue
     public Builder setBlockSize(int blockSize) {
       checkArgument(blockSize > 0, "blockSize (%s) must be positive", blockSize);
       this.blockSize = blockSize;
@@ -429,16 +478,17 @@ public final class Configuration {
      * Sets the maximum size (in bytes) for the file system's in-memory file storage. This maximum
      * size determines the maximum number of blocks that can be allocated to regular files, so it
      * should generally be a multiple of the {@linkplain #setBlockSize(int) block size}. The actual
-     * maximum size will be the nearest multiple of the block size that is less than or equal to
-     * the given size.
+     * maximum size will be the nearest multiple of the block size that is less than or equal to the
+     * given size.
      *
      * <p><b>Note:</b> The in-memory file storage will not be eagerly initialized to this size, so
      * it won't use more memory than is needed for the files you create. Also note that in addition
-     * to this limit, you will of course be limited by the amount of heap space available to the
-     * JVM and the amount of heap used by other objects, both in the file system and elsewhere.
+     * to this limit, you will of course be limited by the amount of heap space available to the JVM
+     * and the amount of heap used by other objects, both in the file system and elsewhere.
      *
      * <p>The default is 4 GB.
      */
+    @CanIgnoreReturnValue
     public Builder setMaxSize(long maxSize) {
       checkArgument(maxSize > 0, "maxSize (%s) must be positive", maxSize);
       this.maxSize = maxSize;
@@ -447,17 +497,18 @@ public final class Configuration {
 
     /**
      * Sets the maximum amount of unused space (in bytes) in the file system's in-memory file
-     * storage that should be cached for reuse. By default, this will be equal to the
-     * {@linkplain #setMaxSize(long) maximum size} of the storage, meaning that all space that is
-     * freed when files are truncated or deleted is cached for reuse. This helps to avoid lots of
-     * garbage collection when creating and deleting many files quickly. This can be set to 0 to
-     * disable caching entirely (all freed blocks become available for garbage collection) or to
-     * some other number to put an upper bound on the maximum amount of unused space the file
-     * system will keep around.
+     * storage that should be cached for reuse. By default, this will be equal to the {@linkplain
+     * #setMaxSize(long) maximum size} of the storage, meaning that all space that is freed when
+     * files are truncated or deleted is cached for reuse. This helps to avoid lots of garbage
+     * collection when creating and deleting many files quickly. This can be set to 0 to disable
+     * caching entirely (all freed blocks become available for garbage collection) or to some other
+     * number to put an upper bound on the maximum amount of unused space the file system will keep
+     * around.
      *
      * <p>Like the maximum size, the actual value will be the closest multiple of the block size
      * that is less than or equal to the given size.
      */
+    @CanIgnoreReturnValue
     public Builder setMaxCacheSize(long maxCacheSize) {
       checkArgument(maxCacheSize >= 0, "maxCacheSize (%s) may not be negative", maxCacheSize);
       this.maxCacheSize = maxCacheSize;
@@ -514,14 +565,14 @@ public final class Configuration {
      * <p>If any other views should be supported, attribute providers for those views must be
      * {@linkplain #addAttributeProvider(AttributeProvider) added}.
      */
+    @CanIgnoreReturnValue
     public Builder setAttributeViews(String first, String... more) {
       this.attributeViews = ImmutableSet.copyOf(Lists.asList(first, more));
       return this;
     }
 
-    /**
-     * Adds an attribute provider for a custom view for the file system to support.
-     */
+    /** Adds an attribute provider for a custom view for the file system to support. */
+    @CanIgnoreReturnValue
     public Builder addAttributeProvider(AttributeProvider provider) {
       checkNotNull(provider);
       if (attributeProviders == null) {
@@ -533,8 +584,8 @@ public final class Configuration {
 
     /**
      * Sets the default value to use for the given file attribute when creating new files. The
-     * attribute must be in the form "view:attribute". The value must be of a type that the
-     * provider for the view accepts.
+     * attribute must be in the form "view:attribute". The value must be of a type that the provider
+     * for the view accepts.
      *
      * <p>For the included attribute views, default values can be set for the following attributes:
      *
@@ -577,6 +628,7 @@ public final class Configuration {
      *   </tr>
      * </table>
      */
+    @CanIgnoreReturnValue
     public Builder setDefaultAttributeValue(String attribute, Object value) {
       checkArgument(
           ATTRIBUTE_PATTERN.matcher(attribute).matches(),
@@ -594,14 +646,26 @@ public final class Configuration {
 
     private static final Pattern ATTRIBUTE_PATTERN = Pattern.compile("[^:]+:[^:]+");
 
+    /**
+     * Sets the {@link FileTimeSource} that will supply the current time for this file system.
+     *
+     * @since 1.3
+     */
+    @CanIgnoreReturnValue
+    public Builder setFileTimeSource(FileTimeSource source) {
+      this.fileTimeSource = checkNotNull(source);
+      return this;
+    }
+
     /**
      * Sets the roots for the file system.
      *
-     * @throws InvalidPathException if any of the given roots is not a valid path for this
-     *     builder's path type
-     * @throws IllegalArgumentException if any of the given roots is a valid path for this
-     *     builder's path type but is not a root path with no name elements
+     * @throws InvalidPathException if any of the given roots is not a valid path for this builder's
+     *     path type
+     * @throws IllegalArgumentException if any of the given roots is a valid path for this builder's
+     *     path type but is not a root path with no name elements
      */
+    @CanIgnoreReturnValue
     public Builder setRoots(String first, String... more) {
       List<String> roots = Lists.asList(first, more);
       for (String root : roots) {
@@ -613,13 +677,14 @@ public final class Configuration {
     }
 
     /**
-     * Sets the path to the working directory for the file system. The working directory must be
-     * an absolute path starting with one of the configured roots.
+     * Sets the path to the working directory for the file system. The working directory must be an
+     * absolute path starting with one of the configured roots.
      *
      * @throws InvalidPathException if the given path is not valid for this builder's path type
      * @throws IllegalArgumentException if the given path is valid for this builder's path type but
      *     is not an absolute path
      */
+    @CanIgnoreReturnValue
     public Builder setWorkingDirectory(String workingDirectory) {
       PathType.ParseResult parseResult = pathType.parsePath(workingDirectory);
       checkArgument(
@@ -634,25 +699,31 @@ public final class Configuration {
      * Sets the given features to be supported by the file system. Any features not provided here
      * will not be supported.
      */
+    @CanIgnoreReturnValue
     public Builder setSupportedFeatures(Feature... features) {
       supportedFeatures = Sets.immutableEnumSet(Arrays.asList(features));
       return this;
     }
 
     /**
-     * Sets the configuration that {@link WatchService} instances created by the file system
-     * should use. The default configuration polls watched directories for changes every 5 seconds.
+     * Sets the configuration that {@link WatchService} instances created by the file system should
+     * use. The default configuration polls watched directories for changes every 5 seconds.
      *
      * @since 1.1
      */
+    @CanIgnoreReturnValue
     public Builder setWatchServiceConfiguration(WatchServiceConfiguration config) {
       this.watchServiceConfig = checkNotNull(config);
       return this;
     }
 
-    /**
-     * Creates a new immutable configuration object from this builder.
-     */
+    @CanIgnoreReturnValue
+    private Builder setDisplayName(String displayName) {
+      this.displayName = checkNotNull(displayName);
+      return this;
+    }
+
+    /** Creates a new immutable configuration object from this builder. */
     public Configuration build() {
       return new Configuration(this);
     }
