@@ -28,7 +28,7 @@ import com.google.common.base.Supplier;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSortedSet;
 import com.google.common.collect.Lists;
-
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import java.io.IOException;
 import java.nio.file.CopyOption;
 import java.nio.file.DirectoryNotEmptyException;
@@ -43,14 +43,14 @@ import java.nio.file.SecureDirectoryStream;
 import java.nio.file.attribute.BasicFileAttributes;
 import java.nio.file.attribute.FileAttribute;
 import java.nio.file.attribute.FileAttributeView;
+import java.nio.file.attribute.FileTime;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
-
-import javax.annotation.Nullable;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * View of a file system with a specific working directory. As all file system operations need to
@@ -70,9 +70,7 @@ final class FileSystemView {
   private final Directory workingDirectory;
   private final JimfsPath workingDirectoryPath;
 
-  /**
-   * Creates a new file system view.
-   */
+  /** Creates a new file system view. */
   public FileSystemView(
       JimfsFileStore store, Directory workingDirectory, JimfsPath workingDirectoryPath) {
     this.store = checkNotNull(store);
@@ -80,20 +78,20 @@ final class FileSystemView {
     this.workingDirectoryPath = checkNotNull(workingDirectoryPath);
   }
 
-  /**
-   * Returns whether or not this view and the given view belong to the same file system.
-   */
+  /** Returns whether or not this view and the given view belong to the same file system. */
   private boolean isSameFileSystem(FileSystemView other) {
     return store == other.store;
   }
 
-  /**
-   * Returns the file system state.
-   */
+  /** Returns the file system state. */
   public FileSystemState state() {
     return store.state();
   }
 
+  private FileTime now() {
+    return state().now();
+  }
+
   /**
    * Returns the path of the working directory at the time this view was created. Does not reflect
    * changes to the path caused by the directory being moved.
@@ -102,9 +100,7 @@ final class FileSystemView {
     return workingDirectoryPath;
   }
 
-  /**
-   * Attempt to look up the file at the given path.
-   */
+  /** Attempt to look up the file at the given path. */
   DirectoryEntry lookUpWithLock(JimfsPath path, Set<? super LinkOption> options)
       throws IOException {
     store.readLock().lock();
@@ -115,18 +111,16 @@ final class FileSystemView {
     }
   }
 
-  /**
-   * Looks up the file at the given path without locking.
-   */
+  /** Looks up the file at the given path without locking. */
   private DirectoryEntry lookUp(JimfsPath path, Set<? super LinkOption> options)
       throws IOException {
     return store.lookUp(workingDirectory, path, options);
   }
 
   /**
-   * Creates a new directory stream for the directory located by the given path. The given
-   * {@code basePathForStream} is that base path that the returned stream will use. This will be
-   * the same as {@code dir} except for streams created relative to another secure stream.
+   * Creates a new directory stream for the directory located by the given path. The given {@code
+   * basePathForStream} is that base path that the returned stream will use. This will be the same
+   * as {@code dir} except for streams created relative to another secure stream.
    */
   public DirectoryStream<Path> newDirectoryStream(
       JimfsPath dir,
@@ -134,9 +128,7 @@ final class FileSystemView {
       Set<? super LinkOption> options,
       JimfsPath basePathForStream)
       throws IOException {
-    Directory file = (Directory) lookUpWithLock(dir, options)
-        .requireDirectory(dir)
-        .file();
+    Directory file = (Directory) lookUpWithLock(dir, options).requireDirectory(dir).file();
     FileSystemView view = new FileSystemView(store, file, basePathForStream);
     JimfsSecureDirectoryStream stream = new JimfsSecureDirectoryStream(view, filter, state());
     return store.supportsFeature(Feature.SECURE_DIRECTORY_STREAM)
@@ -144,14 +136,12 @@ final class FileSystemView {
         : new DowngradedDirectoryStream(stream);
   }
 
-  /**
-   * Snapshots the entries of the working directory of this view.
-   */
+  /** Snapshots the entries of the working directory of this view. */
   public ImmutableSortedSet<Name> snapshotWorkingDirectoryEntries() {
     store.readLock().lock();
     try {
       ImmutableSortedSet<Name> names = workingDirectory.snapshot();
-      workingDirectory.updateAccessTime();
+      workingDirectory.setLastAccessTime(now());
       return names;
     } finally {
       store.readLock().unlock();
@@ -162,14 +152,12 @@ final class FileSystemView {
    * Returns a snapshot mapping the names of each file in the directory at the given path to the
    * last modified time of that file.
    */
-  public ImmutableMap<Name, Long> snapshotModifiedTimes(JimfsPath path) throws IOException {
-    ImmutableMap.Builder<Name, Long> modifiedTimes = ImmutableMap.builder();
+  public ImmutableMap<Name, FileTime> snapshotModifiedTimes(JimfsPath path) throws IOException {
+    ImmutableMap.Builder<Name, FileTime> modifiedTimes = ImmutableMap.builder();
 
     store.readLock().lock();
     try {
-      Directory dir = (Directory) lookUp(path, Options.FOLLOW_LINKS)
-          .requireDirectory(path)
-          .file();
+      Directory dir = (Directory) lookUp(path, Options.FOLLOW_LINKS).requireDirectory(path).file();
       // TODO(cgdecker): Investigate whether WatchServices should keep a reference to the actual
       // directory when SecureDirectoryStream is supported rather than looking up the directory
       // each time the WatchService polls
@@ -207,8 +195,8 @@ final class FileSystemView {
   }
 
   /**
-   * Gets the {@linkplain Path#toRealPath(LinkOption...) real path} to the file located by the
-   * given path.
+   * Gets the {@linkplain Path#toRealPath(LinkOption...) real path} to the file located by the given
+   * path.
    */
   public JimfsPath toRealPath(
       JimfsPath path, PathService pathService, Set<? super LinkOption> options) throws IOException {
@@ -239,6 +227,7 @@ final class FileSystemView {
    * Creates a new directory at the given path. The given attributes will be set on the new file if
    * possible.
    */
+  @CanIgnoreReturnValue
   public Directory createDirectory(JimfsPath path, FileAttribute<?>... attrs) throws IOException {
     return (Directory) createFile(path, store.directoryCreator(), true, attrs);
   }
@@ -247,6 +236,7 @@ final class FileSystemView {
    * Creates a new symbolic link at the given path with the given target. The given attributes will
    * be set on the new file if possible.
    */
+  @CanIgnoreReturnValue
   public SymbolicLink createSymbolicLink(
       JimfsPath path, JimfsPath target, FileAttribute<?>... attrs) throws IOException {
     if (!store.supportsFeature(Feature.SYMBOLIC_LINKS)) {
@@ -257,8 +247,8 @@ final class FileSystemView {
 
   /**
    * Creates a new file at the given path if possible, using the given supplier to create the file.
-   * Returns the new file. If {@code allowExisting} is {@code true} and a file already exists at
-   * the given path, returns that file. Otherwise, throws {@link FileAlreadyExistsException}.
+   * Returns the new file. If {@code allowExisting} is {@code true} and a file already exists at the
+   * given path, returns that file. Otherwise, throws {@link FileAlreadyExistsException}.
    */
   private File createFile(
       JimfsPath path,
@@ -289,7 +279,7 @@ final class FileSystemView {
       File newFile = fileCreator.get();
       store.setInitialAttributes(newFile, attrs);
       parent.link(path.name(), newFile);
-      parent.updateModifiedTime();
+      parent.setLastModifiedTime(now());
       return newFile;
     } finally {
       store.writeLock().unlock();
@@ -323,8 +313,7 @@ final class FileSystemView {
    * Looks up the regular file at the given path, throwing an exception if the file isn't a regular
    * file. Returns null if the file did not exist.
    */
-  @Nullable
-  private RegularFile lookUpRegularFile(JimfsPath path, Set<OpenOption> options)
+  private @Nullable RegularFile lookUpRegularFile(JimfsPath path, Set<OpenOption> options)
       throws IOException {
     store.readLock().lock();
     try {
@@ -343,9 +332,7 @@ final class FileSystemView {
     }
   }
 
-  /**
-   * Gets or creates a new regular file with a write lock (assuming the file does not exist).
-   */
+  /** Gets or creates a new regular file with a write lock (assuming the file does not exist). */
   private RegularFile getOrCreateRegularFileWithWriteLock(
       JimfsPath path, Set<OpenOption> options, FileAttribute<?>[] attrs) throws IOException {
     store.writeLock().lock();
@@ -382,18 +369,15 @@ final class FileSystemView {
     return file;
   }
 
-  /**
-   * Returns the target of the symbolic link at the given path.
-   */
+  /** Returns the target of the symbolic link at the given path. */
   public JimfsPath readSymbolicLink(JimfsPath path) throws IOException {
     if (!store.supportsFeature(Feature.SYMBOLIC_LINKS)) {
       throw new UnsupportedOperationException();
     }
 
     SymbolicLink symbolicLink =
-        (SymbolicLink) lookUpWithLock(path, Options.NOFOLLOW_LINKS)
-            .requireSymbolicLink(path)
-            .file();
+        (SymbolicLink)
+            lookUpWithLock(path, Options.NOFOLLOW_LINKS).requireSymbolicLink(path).file();
 
     return symbolicLink.target();
   }
@@ -436,10 +420,7 @@ final class FileSystemView {
     try {
       // we do want to follow links when finding the existing file
       File existingFile =
-          existingView
-              .lookUp(existing, Options.FOLLOW_LINKS)
-              .requireExists(existing)
-              .file();
+          existingView.lookUp(existing, Options.FOLLOW_LINKS).requireExists(existing).file();
       if (!existingFile.isRegularFile()) {
         throw new FileSystemException(
             link.toString(), existing.toString(), "can't link: not a regular file");
@@ -449,15 +430,13 @@ final class FileSystemView {
           lookUp(link, Options.NOFOLLOW_LINKS).requireDoesNotExist(link).directory();
 
       linkParent.link(linkName, existingFile);
-      linkParent.updateModifiedTime();
+      linkParent.setLastModifiedTime(now());
     } finally {
       store.writeLock().unlock();
     }
   }
 
-  /**
-   * Deletes the file at the given absolute path.
-   */
+  /** Deletes the file at the given absolute path. */
   public void deleteFile(JimfsPath path, DeleteMode deleteMode) throws IOException {
     store.writeLock().lock();
     try {
@@ -468,9 +447,7 @@ final class FileSystemView {
     }
   }
 
-  /**
-   * Deletes the given directory entry from its parent directory.
-   */
+  /** Deletes the given directory entry from its parent directory. */
   private void delete(DirectoryEntry entry, DeleteMode deleteMode, JimfsPath pathForException)
       throws IOException {
     Directory parent = entry.directory();
@@ -478,32 +455,22 @@ final class FileSystemView {
 
     checkDeletable(file, deleteMode, pathForException);
     parent.unlink(entry.name());
-    parent.updateModifiedTime();
+    parent.setLastModifiedTime(now());
 
     file.deleted();
   }
 
-  /**
-   * Mode for deleting. Determines what types of files can be deleted.
-   */
+  /** Mode for deleting. Determines what types of files can be deleted. */
   public enum DeleteMode {
-    /**
-     * Delete any file.
-     */
+    /** Delete any file. */
     ANY,
-    /**
-     * Only delete non-directory files.
-     */
+    /** Only delete non-directory files. */
     NON_DIRECTORY_ONLY,
-    /**
-     * Only delete directory files.
-     */
+    /** Only delete directory files. */
     DIRECTORY_ONLY
   }
 
-  /**
-   * Checks that the given file can be deleted, throwing an exception if it can't.
-   */
+  /** Checks that the given file can be deleted, throwing an exception if it can't. */
   private void checkDeletable(File file, DeleteMode mode, Path path) throws IOException {
     if (file.isRootDirectory()) {
       throw new FileSystemException(path.toString(), null, "can't delete root directory");
@@ -527,18 +494,14 @@ final class FileSystemView {
     }
   }
 
-  /**
-   * Checks that given directory is empty, throwing {@link DirectoryNotEmptyException} if not.
-   */
+  /** Checks that given directory is empty, throwing {@link DirectoryNotEmptyException} if not. */
   private void checkEmpty(Directory dir, Path pathForException) throws FileSystemException {
     if (!dir.isEmpty()) {
       throw new DirectoryNotEmptyException(pathForException.toString());
     }
   }
 
-  /**
-   * Copies or moves the file at the given source path to the given dest path.
-   */
+  /** Copies or moves the file at the given source path to the given dest path. */
   public void copy(
       JimfsPath source,
       FileSystemView destView,
@@ -589,10 +552,10 @@ final class FileSystemView {
       if (move && sameFileSystem) {
         // Real move on the same file system.
         sourceParent.unlink(source.name());
-        sourceParent.updateModifiedTime();
+        sourceParent.setLastModifiedTime(now());
 
         destParent.link(dest.name(), sourceFile);
-        destParent.updateModifiedTime();
+        destParent.setLastModifiedTime(now());
       } else {
         // Doing a copy OR a move to a different file system, which must be implemented by copy and
         // delete.
@@ -615,7 +578,7 @@ final class FileSystemView {
         // Copy the file, but don't copy its content while we're holding the file store locks.
         copyFile = destView.store.copyWithoutContent(sourceFile, attributeCopyOption);
         destParent.link(dest.name(), copyFile);
-        destParent.updateModifiedTime();
+        destParent.setLastModifiedTime(now());
 
         // In order for the copy to be atomic (not strictly necessary, but seems preferable since
         // we can) lock both source and copy files before leaving the file store locks. This
@@ -680,9 +643,7 @@ final class FileSystemView {
     }
   }
 
-  /**
-   * Checks that source is not an ancestor of dest, throwing an exception if it is.
-   */
+  /** Checks that source is not an ancestor of dest, throwing an exception if it is. */
   private void checkNotAncestor(File source, Directory destParent, FileSystemView destView)
       throws IOException {
     // if dest is not in the same file system, it couldn't be in source's subdirectories
@@ -722,8 +683,8 @@ final class FileSystemView {
   }
 
   /**
-   * Unlocks source and copy files after copying content. Also closes the source file so its
-   * content can be deleted if it was deleted.
+   * Unlocks source and copy files after copying content. Also closes the source file so its content
+   * can be deleted if it was deleted.
    */
   private void unlockSourceAndCopy(File sourceFile, File copyFile) {
     ReadWriteLock sourceLock = sourceFile.contentLock();
@@ -737,64 +698,46 @@ final class FileSystemView {
     sourceFile.closed();
   }
 
-  /**
-   * Returns a file attribute view using the given lookup callback.
-   */
-  @Nullable
-  public <V extends FileAttributeView> V getFileAttributeView(FileLookup lookup, Class<V> type) {
+  /** Returns a file attribute view using the given lookup callback. */
+  public <V extends FileAttributeView> @Nullable V getFileAttributeView(
+      FileLookup lookup, Class<V> type) {
     return store.getFileAttributeView(lookup, type);
   }
 
-  /**
-   * Returns a file attribute view for the given path in this view.
-   */
-  @Nullable
-  public <V extends FileAttributeView> V getFileAttributeView(
+  /** Returns a file attribute view for the given path in this view. */
+  public <V extends FileAttributeView> @Nullable V getFileAttributeView(
       final JimfsPath path, Class<V> type, final Set<? super LinkOption> options) {
     return store.getFileAttributeView(
         new FileLookup() {
           @Override
           public File lookup() throws IOException {
-            return lookUpWithLock(path, options)
-                .requireExists(path)
-                .file();
+            return lookUpWithLock(path, options).requireExists(path).file();
           }
         },
         type);
   }
 
-  /**
-   * Reads attributes of the file located by the given path in this view as an object.
-   */
+  /** Reads attributes of the file located by the given path in this view as an object. */
   public <A extends BasicFileAttributes> A readAttributes(
       JimfsPath path, Class<A> type, Set<? super LinkOption> options) throws IOException {
-    File file = lookUpWithLock(path, options)
-        .requireExists(path)
-        .file();
+    File file = lookUpWithLock(path, options).requireExists(path).file();
     return store.readAttributes(file, type);
   }
 
-  /**
-   * Reads attributes of the file located by the given path in this view as a map.
-   */
+  /** Reads attributes of the file located by the given path in this view as a map. */
   public ImmutableMap<String, Object> readAttributes(
       JimfsPath path, String attributes, Set<? super LinkOption> options) throws IOException {
-    File file = lookUpWithLock(path, options)
-        .requireExists(path)
-        .file();
+    File file = lookUpWithLock(path, options).requireExists(path).file();
     return store.readAttributes(file, attributes);
   }
 
   /**
-   * Sets the given attribute to the given value on the file located by the given path in this
-   * view.
+   * Sets the given attribute to the given value on the file located by the given path in this view.
    */
   public void setAttribute(
       JimfsPath path, String attribute, Object value, Set<? super LinkOption> options)
       throws IOException {
-    File file = lookUpWithLock(path, options)
-        .requireExists(path)
-        .file();
+    File file = lookUpWithLock(path, options).requireExists(path).file();
     store.setAttribute(file, attribute, value);
   }
 }
