@@ -23,12 +23,13 @@ import static com.google.common.jimfs.Util.nextPowerOf2;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.primitives.UnsignedBytes;
-
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.channels.ReadableByteChannel;
 import java.nio.channels.WritableByteChannel;
+import java.nio.file.attribute.FileTime;
 import java.util.Arrays;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
@@ -53,15 +54,19 @@ final class RegularFile extends File {
 
   private long size;
 
-  /**
-   * Creates a new regular file with the given ID and using the given disk.
-   */
-  public static RegularFile create(int id, HeapDisk disk) {
-    return new RegularFile(id, disk, new byte[32][], 0, 0);
+  /** Creates a new regular file with the given ID and using the given disk. */
+  public static RegularFile create(int id, FileTime creationTime, HeapDisk disk) {
+    return new RegularFile(id, creationTime, disk, new byte[32][], 0, 0);
   }
 
-  RegularFile(int id, HeapDisk disk, byte[][] blocks, int blockCount, long size) {
-    super(id);
+  RegularFile(
+      int id,
+      FileTime creationTime,
+      HeapDisk disk,
+      byte[][] blocks,
+      int blockCount,
+      long size) {
+    super(id, creationTime);
     this.disk = checkNotNull(disk);
     this.blocks = checkNotNull(blocks);
     this.blockCount = blockCount;
@@ -73,16 +78,12 @@ final class RegularFile extends File {
   private int openCount = 0;
   private boolean deleted = false;
 
-  /**
-   * Returns the read lock for this file.
-   */
+  /** Returns the read lock for this file. */
   public Lock readLock() {
     return lock.readLock();
   }
 
-  /**
-   * Returns the write lock for this file.
-   */
+  /** Returns the write lock for this file. */
   public Lock writeLock() {
     return lock.writeLock();
   }
@@ -95,16 +96,12 @@ final class RegularFile extends File {
     }
   }
 
-  /**
-   * Returns the number of blocks this file contains.
-   */
+  /** Returns the number of blocks this file contains. */
   int blockCount() {
     return blockCount;
   }
 
-  /**
-   * Copies the last {@code count} blocks from this file to the end of the given target file.
-   */
+  /** Copies the last {@code count} blocks from this file to the end of the given target file. */
   void copyBlocksTo(RegularFile target, int count) {
     int start = blockCount - count;
     int targetEnd = target.blockCount + count;
@@ -114,33 +111,25 @@ final class RegularFile extends File {
     target.blockCount = targetEnd;
   }
 
-  /**
-   * Transfers the last {@code count} blocks from this file to the end of the given target file.
-   */
+  /** Transfers the last {@code count} blocks from this file to the end of the given target file. */
   void transferBlocksTo(RegularFile target, int count) {
     copyBlocksTo(target, count);
     truncateBlocks(blockCount - count);
   }
 
-  /**
-   * Truncates the blocks of this file to the given block count.
-   */
+  /** Truncates the blocks of this file to the given block count. */
   void truncateBlocks(int count) {
     clear(blocks, count, blockCount - count);
     blockCount = count;
   }
 
-  /**
-   * Adds the given block to the end of this file.
-   */
+  /** Adds the given block to the end of this file. */
   void addBlock(byte[] block) {
     expandIfNecessary(blockCount + 1);
     blocks[blockCount++] = block;
   }
 
-  /**
-   * Gets the block at the given index in this file.
-   */
+  /** Gets the block at the given index in this file. */
   @VisibleForTesting
   byte[] getBlock(int index) {
     return blocks[index];
@@ -149,8 +138,8 @@ final class RegularFile extends File {
   // end of lower-level methods dealing with the blocks array
 
   /**
-   * Gets the current size of this file in bytes. Does not do locking, so should only be called
-   * when holding a lock.
+   * Gets the current size of this file in bytes. Does not do locking, so should only be called when
+   * holding a lock.
    */
   public long sizeWithoutLocking() {
     return size;
@@ -169,9 +158,9 @@ final class RegularFile extends File {
   }
 
   @Override
-  RegularFile copyWithoutContent(int id) {
+  RegularFile copyWithoutContent(int id, FileTime creationTime) {
     byte[][] copyBlocks = new byte[Math.max(blockCount * 2, 32)][];
-    return new RegularFile(id, disk, copyBlocks, 0, size);
+    return new RegularFile(id, creationTime, disk, copyBlocks, 0, size);
   }
 
   @Override
@@ -207,8 +196,8 @@ final class RegularFile extends File {
   }
 
   /**
-   * Marks this file as deleted. If there are no streams or channels open to the file, its
-   * contents are deleted if necessary.
+   * Marks this file as deleted. If there are no streams or channels open to the file, its contents
+   * are deleted if necessary.
    */
   @Override
   public synchronized void deleted() {
@@ -231,11 +220,12 @@ final class RegularFile extends File {
 
   /**
    * Truncates this file to the given {@code size}. If the given size is less than the current size
-   * of this file, the size of the file is reduced to the given size and any bytes beyond that
-   * size are lost. If the given size is greater than the current size of the file, this method
-   * does nothing. Returns {@code true} if this file was modified by the call (its size changed)
-   * and {@code false} otherwise.
+   * of this file, the size of the file is reduced to the given size and any bytes beyond that size
+   * are lost. If the given size is greater than the current size of the file, this method does
+   * nothing. Returns {@code true} if this file was modified by the call (its size changed) and
+   * {@code false} otherwise.
    */
+  @CanIgnoreReturnValue
   public boolean truncate(long size) {
     if (size >= this.size) {
       return false;
@@ -253,9 +243,7 @@ final class RegularFile extends File {
     return true;
   }
 
-  /**
-   * Prepares for a write of len bytes starting at position pos.
-   */
+  /** Prepares for a write of len bytes starting at position pos. */
   private void prepareForWrite(long pos, long len) throws IOException {
     long end = pos + len;
 
@@ -289,12 +277,13 @@ final class RegularFile extends File {
   }
 
   /**
-   * Writes the given byte to this file at position {@code pos}. {@code pos} may be greater than
-   * the current size of this file, in which case this file is resized and all bytes between the
-   * current size and {@code pos} are set to 0. Returns the number of bytes written.
+   * Writes the given byte to this file at position {@code pos}. {@code pos} may be greater than the
+   * current size of this file, in which case this file is resized and all bytes between the current
+   * size and {@code pos} are set to 0. Returns the number of bytes written.
    *
    * @throws IOException if the file needs more blocks but the disk is full
    */
+  @CanIgnoreReturnValue
   public int write(long pos, byte b) throws IOException {
     prepareForWrite(pos, 1);
 
@@ -312,11 +301,12 @@ final class RegularFile extends File {
   /**
    * Writes {@code len} bytes starting at offset {@code off} in the given byte array to this file
    * starting at position {@code pos}. {@code pos} may be greater than the current size of this
-   * file, in which case this file is resized and all bytes between the current size and {@code
-   * pos} are set to 0. Returns the number of bytes written.
+   * file, in which case this file is resized and all bytes between the current size and {@code pos}
+   * are set to 0. Returns the number of bytes written.
    *
    * @throws IOException if the file needs more blocks but the disk is full
    */
+  @CanIgnoreReturnValue
   public int write(long pos, byte[] b, int off, int len) throws IOException {
     prepareForWrite(pos, len);
 
@@ -352,12 +342,13 @@ final class RegularFile extends File {
 
   /**
    * Writes all available bytes from buffer {@code buf} to this file starting at position {@code
-   * pos}. {@code pos} may be greater than the current size of this file, in which case this file
-   * is resized and all bytes between the current size and {@code pos} are set to 0. Returns the
-   * number of bytes written.
+   * pos}. {@code pos} may be greater than the current size of this file, in which case this file is
+   * resized and all bytes between the current size and {@code pos} are set to 0. Returns the number
+   * of bytes written.
    *
    * @throws IOException if the file needs more blocks but the disk is full
    */
+  @CanIgnoreReturnValue
   public int write(long pos, ByteBuffer buf) throws IOException {
     int len = buf.remaining();
 
@@ -388,13 +379,14 @@ final class RegularFile extends File {
   }
 
   /**
-   * Writes all available bytes from each buffer in {@code bufs}, in order, to this file starting
-   * at position {@code pos}. {@code pos} may be greater than the current size of this file, in
-   * which case this file is resized and all bytes between the current size and {@code pos} are set
-   * to 0. Returns the number of bytes written.
+   * Writes all available bytes from each buffer in {@code bufs}, in order, to this file starting at
+   * position {@code pos}. {@code pos} may be greater than the current size of this file, in which
+   * case this file is resized and all bytes between the current size and {@code pos} are set to 0.
+   * Returns the number of bytes written.
    *
    * @throws IOException if the file needs more blocks but the disk is full
    */
+  @CanIgnoreReturnValue
   public long write(long pos, Iterable<ByteBuffer> bufs) throws IOException {
     long start = pos;
     for (ByteBuffer buf : bufs) {
@@ -406,70 +398,66 @@ final class RegularFile extends File {
   /**
    * Transfers up to {@code count} bytes from the given channel to this file starting at position
    * {@code pos}. Returns the number of bytes transferred. If {@code pos} is greater than the
-   * current size of this file, the file is truncated up to size {@code pos} before writing.
+   * current size of this file, then no bytes are transferred.
    *
    * @throws IOException if the file needs more blocks but the disk is full or if reading from src
    *     throws an exception
    */
-  public long transferFrom(ReadableByteChannel src, long pos, long count) throws IOException {
-    prepareForWrite(pos, 0); // don't assume the full count bytes will be written
-
-    if (count == 0) {
+  public long transferFrom(ReadableByteChannel src, long startPos, long count) throws IOException {
+    if (count == 0
+        // Unlike the write() methods, attempting to transfer to a position that is greater than the
+        // current file size simply does nothing.
+        || startPos > size) {
       return 0;
     }
 
     long remaining = count;
+    long currentPos = startPos;
 
-    int blockIndex = blockIndex(pos);
-    byte[] block = blockForWrite(blockIndex);
-    int off = offsetInBlock(pos);
+    int blockIndex = blockIndex(startPos);
+    int off = offsetInBlock(startPos);
 
-    ByteBuffer buf = ByteBuffer.wrap(block, off, length(off, remaining));
+    outer:
+    while (remaining > 0) {
+      byte[] block = blockForWrite(blockIndex);
 
-    long currentPos = pos;
-    int read = 0;
-    while (buf.hasRemaining()) {
-      read = src.read(buf);
-      if (read == -1) {
-        break;
-      }
-
-      currentPos += read;
-      remaining -= read;
-    }
-
-    // update size before trying to get next block in case the disk is out of space
-    if (currentPos > size) {
-      size = currentPos;
-    }
-
-    if (read != -1) {
-      outer:
-      while (remaining > 0) {
-        block = blockForWrite(++blockIndex);
-
-        buf = ByteBuffer.wrap(block, 0, length(remaining));
-        while (buf.hasRemaining()) {
-          read = src.read(buf);
-          if (read == -1) {
-            break outer;
+      ByteBuffer buf = ByteBuffer.wrap(block, off, length(off, remaining));
+      while (buf.hasRemaining()) {
+        int read = src.read(buf);
+        // Note: we stop if we read 0 bytes from the src; even though the src is not at EOF, the
+        // spec of transferFrom is to stop immediately when reading from a non-blocking channel that
+        // has no bytes available rather than continuing until it reaches EOF. This makes sense
+        // because we'd otherwise just spin attempting to read bytes from the src repeatedly.
+        if (read < 1) {
+          if (currentPos >= size && buf.position() == 0) {
+            // The current position is at or beyond the end of file (prior to transfer start) and
+            // the current buffer position is 0. This means that a new block must have just been
+            // allocated to hold any potential transferred bytes, but no bytes were transferred to
+            // it because the src had no remaining bytes. So we need to de-allocate that block.
+            // It's possible that it would be preferable to always transfer to a temporary block
+            // first and then copy that block to a newly allocated block when it's full or src
+            // doesn't have any further bytes. Then if we hadn't read anything into the temporary
+            // block, we could simply discard it. But I think this scenario is likely rare enough
+            // that it's fine to temporarily allocate a block that _might_ not get used.
+            disk.free(this, 1);
           }
-
-          currentPos += read;
-          remaining -= read;
+          break outer;
         }
 
-        if (currentPos > size) {
-          size = currentPos;
-        }
+        currentPos += read;
+        remaining -= read;
       }
+
+      // Current write block is full
+      blockIndex++;
+      off = 0;
     }
 
     if (currentPos > size) {
       size = currentPos;
     }
 
-    return currentPos - pos;
+    return currentPos - startPos;
   }
 
   /**
@@ -572,10 +560,10 @@ final class RegularFile extends File {
 
   /**
    * Transfers up to {@code count} bytes to the given channel starting at position {@code pos} in
-   * this file. Returns the number of bytes transferred, possibly 0. Note that unlike all other
-   * read methods in this class, this method does not return -1 if {@code pos} is greater than or
-   * equal to the current size. This for consistency with {@link FileChannel#transferTo}, which
-   * this method is primarily intended as an implementation of.
+   * this file. Returns the number of bytes transferred, possibly 0. Note that unlike all other read
+   * methods in this class, this method does not return -1 if {@code pos} is greater than or equal
+   * to the current size. This for consistency with {@link FileChannel#transferTo}, which this
+   * method is primarily intended as an implementation of.
    */
   public long transferTo(long pos, long count, WritableByteChannel dest) throws IOException {
     long bytesToRead = bytesToRead(pos, count);
@@ -591,7 +579,7 @@ final class RegularFile extends File {
       while (buf.hasRemaining()) {
         remaining -= dest.write(buf);
       }
-      buf.clear();
+      Java8Compatibility.clear(buf);
 
       while (remaining > 0) {
         int index = ++blockIndex;
@@ -601,16 +589,14 @@ final class RegularFile extends File {
         while (buf.hasRemaining()) {
           remaining -= dest.write(buf);
         }
-        buf.clear();
+        Java8Compatibility.clear(buf);
       }
     }
 
     return Math.max(bytesToRead, 0); // don't return -1 for this method
   }
 
-  /**
-   * Gets the block at the given index, expanding to create the block if necessary.
-   */
+  /** Gets the block at the given index, expanding to create the block if necessary. */
   private byte[] blockForWrite(int index) throws IOException {
     if (index >= blockCount) {
       int additionalBlocksNeeded = index - blockCount + 1;
@@ -648,29 +634,22 @@ final class RegularFile extends File {
     return Math.min(available, max);
   }
 
-  /**
-   * Zeroes len bytes in the given block starting at the given offset. Returns len.
-   */
+  /** Zeroes len bytes in the given block starting at the given offset. Returns len. */
   private static int zero(byte[] block, int offset, int len) {
     Util.zero(block, offset, len);
     return len;
   }
 
-  /**
-   * Puts the given slice of the given array at the given offset in the given block.
-   */
+  /** Puts the given slice of the given array at the given offset in the given block. */
   private static int put(byte[] block, int offset, byte[] b, int off, int len) {
     System.arraycopy(b, off, block, offset, len);
     return len;
   }
 
-  /**
-   * Puts the contents of the given byte buffer at the given offset in the given block.
-   */
-  private static int put(byte[] block, int offset, ByteBuffer buf) {
+  /** Puts the contents of the given byte buffer at the given offset in the given block. */
+  private static void put(byte[] block, int offset, ByteBuffer buf) {
     int len = Math.min(block.length - offset, buf.remaining());
     buf.get(block, offset, len);
-    return len;
   }
 
   /**
@@ -682,9 +661,7 @@ final class RegularFile extends File {
     return len;
   }
 
-  /**
-   * Reads len bytes starting at the given offset in the given block into the given byte buffer.
-   */
+  /** Reads len bytes starting at the given offset in the given block into the given byte buffer. */
   private static int get(byte[] block, int offset, ByteBuffer buf, int len) {
     buf.put(block, offset, len);
     return len;
