@@ -20,7 +20,7 @@ import static com.google.common.jimfs.TestUtils.assertNotEquals;
 import static com.google.common.jimfs.TestUtils.buffer;
 import static com.google.common.jimfs.TestUtils.bytes;
 import static com.google.common.jimfs.TestUtils.regularFile;
-import static com.google.common.truth.Truth.assertThat;
+import static com.google.common.truth.Truth.assertWithMessage;
 import static java.nio.file.StandardOpenOption.APPEND;
 import static java.nio.file.StandardOpenOption.READ;
 import static java.nio.file.StandardOpenOption.WRITE;
@@ -37,11 +37,6 @@ import com.google.common.testing.NullPointerTester;
 import com.google.common.util.concurrent.Runnables;
 import com.google.common.util.concurrent.SettableFuture;
 import com.google.common.util.concurrent.Uninterruptibles;
-
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.JUnit4;
-
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.AsynchronousCloseException;
@@ -53,6 +48,7 @@ import java.nio.channels.FileLockInterruptionException;
 import java.nio.channels.NonReadableChannelException;
 import java.nio.channels.NonWritableChannelException;
 import java.nio.file.OpenOption;
+import java.nio.file.attribute.FileTime;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.Callable;
@@ -61,11 +57,14 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.JUnit4;
 
 /**
  * Most of the behavior of {@link JimfsFileChannel} is handled by the {@link RegularFile}
- * implementations, so the thorough tests of that are in {@link RegularFileTest}. This mostly
- * tests interactions with the file and channel positions.
+ * implementations, so the thorough tests of that are in {@link RegularFileTest}. This mostly tests
+ * interactions with the file and channel positions.
  *
  * @author Colin Decker
  */
@@ -76,7 +75,7 @@ public class JimfsFileChannelTest {
     return new JimfsFileChannel(
         file,
         Options.getOptionsForChannel(ImmutableSet.copyOf(options)),
-        new FileSystemState(Runnables.doNothing()));
+        new FileSystemState(new FakeFileTimeSource(), Runnables.doNothing()));
   }
 
   @Test
@@ -229,76 +228,77 @@ public class JimfsFileChannelTest {
   @Test
   public void testFileTimeUpdates() throws IOException {
     RegularFile file = regularFile(10);
+    FakeFileTimeSource fileTimeSource = new FakeFileTimeSource();
     FileChannel channel =
         new JimfsFileChannel(
             file,
             ImmutableSet.<OpenOption>of(READ, WRITE),
-            new FileSystemState(Runnables.doNothing()));
+            new FileSystemState(fileTimeSource, Runnables.doNothing()));
 
-    // accessed
-    long accessTime = file.getLastAccessTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    // accessedTime
+    FileTime accessTime = file.getLastAccessTime();
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.read(ByteBuffer.allocate(10));
     assertNotEquals(accessTime, file.getLastAccessTime());
 
     accessTime = file.getLastAccessTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.read(ByteBuffer.allocate(10), 0);
     assertNotEquals(accessTime, file.getLastAccessTime());
 
     accessTime = file.getLastAccessTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.read(new ByteBuffer[] {ByteBuffer.allocate(10)});
     assertNotEquals(accessTime, file.getLastAccessTime());
 
     accessTime = file.getLastAccessTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.read(new ByteBuffer[] {ByteBuffer.allocate(10)}, 0, 1);
     assertNotEquals(accessTime, file.getLastAccessTime());
 
     accessTime = file.getLastAccessTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.transferTo(0, 10, new ByteBufferChannel(10));
     assertNotEquals(accessTime, file.getLastAccessTime());
 
     // modified
-    long modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    FileTime modifiedTime = file.getLastModifiedTime();
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.write(ByteBuffer.allocate(10));
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
 
     modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.write(ByteBuffer.allocate(10), 0);
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
 
     modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.write(new ByteBuffer[] {ByteBuffer.allocate(10)});
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
 
     modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.write(new ByteBuffer[] {ByteBuffer.allocate(10)}, 0, 1);
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
 
     modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.truncate(0);
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
 
     modifiedTime = file.getLastModifiedTime();
-    Uninterruptibles.sleepUninterruptibly(2, MILLISECONDS);
+    fileTimeSource.advance(2, MILLISECONDS);
 
     channel.transferFrom(new ByteBufferChannel(10), 0, 10);
     assertNotEquals(modifiedTime, file.getLastModifiedTime());
@@ -697,8 +697,8 @@ public class JimfsFileChannelTest {
         future.get();
         fail();
       } catch (ExecutionException expected) {
-        assertThat(expected.getCause())
-            .named("blocking thread exception")
+        assertWithMessage("blocking thread exception")
+            .that(expected.getCause())
             .isInstanceOf(AsynchronousCloseException.class);
       }
     }
@@ -756,8 +756,8 @@ public class JimfsFileChannelTest {
     thread.interrupt();
 
     // get the exception that caused the interrupted operation to terminate
-    assertThat(interruptException.get(200, MILLISECONDS))
-        .named("interrupted thread exception")
+    assertWithMessage("interrupted thread exception")
+        .that(interruptException.get(200, MILLISECONDS))
         .isInstanceOf(ClosedByInterruptException.class);
 
     // check that each other thread got AsynchronousCloseException (since the interrupt, on a
@@ -767,8 +767,8 @@ public class JimfsFileChannelTest {
         future.get();
         fail();
       } catch (ExecutionException expected) {
-        assertThat(expected.getCause())
-            .named("blocking thread exception")
+        assertWithMessage("blocking thread exception")
+            .that(expected.getCause())
             .isInstanceOf(AsynchronousCloseException.class);
       }
     }
@@ -777,9 +777,9 @@ public class JimfsFileChannelTest {
   private static final int BLOCKING_OP_COUNT = 10;
 
   /**
-   * Queues blocking operations on the channel in separate threads using the given executor.
-   * The given latch should have a count of BLOCKING_OP_COUNT to allow the caller wants to wait for
-   * all threads to start executing.
+   * Queues blocking operations on the channel in separate threads using the given executor. The
+   * given latch should have a count of BLOCKING_OP_COUNT to allow the caller wants to wait for all
+   * threads to start executing.
    */
   private List<Future<?>> queueAllBlockingOperations(
       final FileChannel channel, ExecutorService executor, final CountDownLatch startLatch) {
@@ -901,8 +901,7 @@ public class JimfsFileChannelTest {
 
   /**
    * Tests that the methods on the default FileChannel that support InterruptibleChannel behavior
-   * also support it on JimfsFileChannel, by just interrupting the thread before calling the
-   * method.
+   * also support it on JimfsFileChannel, by just interrupting the thread before calling the method.
    */
   @Test
   public void testInterruptedThreads() throws IOException {
@@ -1030,9 +1029,9 @@ public class JimfsFileChannelTest {
   }
 
   /**
-   * Asserts that when the given operation is run on an interrupted thread,
-   * {@code ClosedByInterruptException} is thrown, the channel is closed and the thread is no
-   * longer interrupted.
+   * Asserts that when the given operation is run on an interrupted thread, {@code
+   * ClosedByInterruptException} is thrown, the channel is closed and the thread is no longer
+   * interrupted.
    */
   private static void assertClosedByInterrupt(FileChannelMethod method) throws IOException {
     FileChannel channel = channel(regularFile(10), READ, WRITE);
